# Generated by roxygen2: do not edit by hand

export(Cluster_Gauss_Newton_Bootstrap_method)
export(Cluster_Gauss_Newton_EBE_method)
export(Cluster_Gauss_Newton_method)
export(acceptedApproximateMinimizers)
export(acceptedIndices)
export(acceptedIndices_binary)
export(acceptedMaxSSR)
export(bestApproximateMinimizers)
export(col_quantile)
export(make_ShinyCGNM_doseData)
export(make_ShinyCGNM_observationData)
export(plot_2DprofileLikelihood)
export(plot_Rank_SSR)
export(plot_SSR_parameterValue)
export(plot_SSRsurface)
export(plot_goodnessOfFit)
export(plot_paraDistribution_byHistogram)
export(plot_paraDistribution_byViolinPlots)
export(plot_parameterValue_scatterPlots)
export(plot_profileLikelihood)
export(plot_simulationMatrixWithCI)
export(plot_simulationWithCI)
export(shinyCGNM)
export(suggestInitialLowerRange)
export(suggestInitialUpperRange)
export(table_parameterSummary)
export(table_profileLikelihoodConfidenceInterval)
export(topIndices)
import(MASS)
import(ggplot2)
import(shiny)
import(stats)
import(utils)
